@@ -1,12 +1,16 @@
-use clap::{AppSettings, Clap};
-use std::{ffi::OsStr, fs::File, path::Path, process::exit};
+#![cfg(feature = "cli")]
+
+use clap::Parser;
+use std::ffi::OsStr;
+use std::fs::File;
+use std::path::Path;
+use std::process::exit;
 
 use wallet_pass::sign;
 
 /// Sign an apple wallet pass with a given certificate
-#[derive(Clap, Debug)]
+#[derive(Debug, Parser)]
 #[clap(version = "1.0", author = "Lars Westermann <git@lars-westermann.de>")]
-#[clap(setting = AppSettings::ColoredHelp)]
 struct Opts {
     /// Path to the pass directory
     #[clap(short = 'p', long = "pass")]
@@ -45,11 +49,11 @@ pub fn main() {
     let path = Path::new(&output_path);
     let file = File::create(&path).unwrap();
     if let Err(e) = sign::sign_path(
-        Path::new(&opts.pass_path),
+        &opts.pass_path,
         None,
-        Path::new(&opts.certificate_path),
+        &opts.certificate_path,
         &opts.certificate_password,
-        Path::new(&opts.wwdr_intermediate_certificate_path),
+        &opts.wwdr_intermediate_certificate_path,
         file,
         opts.force_pass_signing,
     ) {
