@@ -41,9 +41,12 @@
 //!     Path::new("Apple Worldwide Developer Relations Certification Authority.pem"),
 //!     Path::new("./StoreCard.pkpass"),
 //! )
-//! .unwrap();  
+//! .unwrap();
 //! ```
 
+#[cfg(feature = "cli")]
+extern crate clap;
+
 mod pass;
 pub use pass::Pass;
 
