@@ -1,9 +1,7 @@
-use std::{
-    fs::File,
-    io::{self, BufReader, Read, Seek, Write},
-    ops::{Deref, DerefMut},
-    path::{Path, PathBuf},
-};
+use std::fs::File;
+use std::io::{self, BufReader, Read, Seek, Write};
+use std::ops::{Deref, DerefMut};
+use std::path::{Path, PathBuf};
 
 use crate::{sign, template::Template};
 
@@ -18,36 +16,35 @@ pub struct Pass {
 
 impl Pass {
     /// Parse a `Pass` instance from the given directory
-    pub fn from_path(pass_path: &Path) -> io::Result<Self> {
-        let mut path_buf = pass_path.to_path_buf();
-        path_buf.push("pass.json");
+    pub fn from_path<P: AsRef<Path>>(pass_path: P) -> io::Result<Self> {
+        let path_buf = pass_path.as_ref().join("pass.json");
 
-        let file = File::open(&path_buf).unwrap();
+        let file = File::open(&path_buf)?;
         let mut file_reader = BufReader::new(file);
         let mut file_buffer = Vec::new();
-        file_reader.read_to_end(&mut file_buffer).unwrap();
+        file_reader.read_to_end(&mut file_buffer)?;
         let template: crate::template::Template = serde_json::from_slice(&file_buffer)?;
 
         Ok(Self {
-            pass_path: pass_path.to_path_buf(),
+            pass_path: pass_path.as_ref().to_path_buf(),
             template,
         })
     }
 
     /// Create a `Pass` instance from the given `Template` and an reference to a directory with image and resource files
-    pub fn from_template(template: &Template, pass_path: &Path) -> Self {
+    pub fn from_template<P: AsRef<Path>>(template: &Template, pass_path: P) -> Self {
         Self {
-            pass_path: pass_path.to_path_buf(),
+            pass_path: pass_path.as_ref().to_path_buf(),
             template: template.clone(),
         }
     }
 
     /// Sign, package and save this `Pass` to writer
-    pub fn export<T>(
+    pub fn export<T, P1: AsRef<Path>, P2: AsRef<Path>>(
         &self,
-        certificate_path: &Path,
+        certificate_path: P1,
         certificate_password: &str,
-        wwdr_intermediate_certificate_path: &Path,
+        wwdr_intermediate_certificate_path: P2,
         writer: T,
     ) -> io::Result<T>
     where
@@ -65,15 +62,14 @@ impl Pass {
     }
 
     /// Sign, package and save this `Pass` to a file
-    pub fn export_to_file(
+    pub fn export_to_file<P1: AsRef<Path>, P2: AsRef<Path>, P3: AsRef<Path>>(
         &self,
-        certificate_path: &Path,
+        certificate_path: P1,
         certificate_password: &str,
-        wwdr_intermediate_certificate_path: &Path,
-        output_path: &Path,
+        wwdr_intermediate_certificate_path: P2,
+        output_path: P3,
     ) -> io::Result<()> {
-        let path = Path::new(output_path);
-        let file = File::create(&path).unwrap();
+        let file = File::create(output_path)?;
         self.export(
             certificate_path,
             certificate_password,
